@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from .util import pretty_ts
 
 
 class BaseEnhancement(object):
@@ -14,6 +15,11 @@ class BaseEnhancement(object):
         raise NotImplementedError()
 
 
+class TimeEnhancement(BaseEnhancement):
+    def process(self, match):
+        match['@timestamp'] = pretty_ts(match['@timestamp'])
+
+
 class DropMatchException(Exception):
     """ ElastAlert will drop a match if this exception type is raised by an enhancement """
     pass
