@@ -1,12 +1,14 @@
 # -*- coding: utf-8 -*-
 import json
 import logging
+
 import requests
-from alerts import Alerter
-from alerts import BasicMatchString
-from util import EAException
-from util import elastalert_logger
-from util import lookup_es_key
+
+from .alerts import Alerter
+from .alerts import BasicMatchString
+from .util import EAException
+from .util import elastalert_logger
+from .util import lookup_es_key
 
 
 class OpsGenieAlerter(Alerter):
@@ -15,23 +17,52 @@ class OpsGenieAlerter(Alerter):
 
     def __init__(self, *args):
         super(OpsGenieAlerter, self).__init__(*args)
-
         self.account = self.rule.get('opsgenie_account')
         self.api_key = self.rule.get('opsgenie_key', 'key')
+        self.default_reciepients = self.rule.get('opsgenie_default_receipients', None)
         self.recipients = self.rule.get('opsgenie_recipients')
+        self.recipients_args = self.rule.get('opsgenie_recipients_args')
+        self.default_teams = self.rule.get('opsgenie_default_teams', None)
         self.teams = self.rule.get('opsgenie_teams')
+        self.teams_args = self.rule.get('opsgenie_teams_args')
         self.tags = self.rule.get('opsgenie_tags', []) + ['ElastAlert', self.rule['name']]
-        self.to_addr = self.rule.get('opsgenie_addr', 'https://api.opsgenie.com/v1/json/alert')
+        self.to_addr = self.rule.get('opsgenie_addr', 'https://api.opsgenie.com/v2/alerts')
         self.custom_message = self.rule.get('opsgenie_message')
         self.opsgenie_subject = self.rule.get('opsgenie_subject')
         self.opsgenie_subject_args = self.rule.get('opsgenie_subject_args')
         self.alias = self.rule.get('opsgenie_alias')
         self.opsgenie_proxy = self.rule.get('opsgenie_proxy', None)
+        self.priority = self.rule.get('opsgenie_priority')
+
+    def _parse_responders(self, responders, responder_args, matches, default_responders):
+        if responder_args:
+            formated_responders = list()
+            responders_values = dict((k, lookup_es_key(matches[0], v)) for k, v in responder_args.items())
+            responders_values = dict((k, v) for k, v in responders_values.items() if v)
+
+            for responder in responders:
+                responder = str(responder)
+                try:
+                    formated_responders.append(responder.format(**responders_values))
+                except KeyError as error:
+                    logging.warn("OpsGenieAlerter: Cannot create responder for OpsGenie Alert. Key not foud: %s. " % (error))
+            if not formated_responders:
+                logging.warn("OpsGenieAlerter: no responders can be formed. Trying the default responder ")
+                if not default_responders:
+                    logging.warn("OpsGenieAlerter: default responder not set. Falling back")
+                    formated_responders = responders
+                else:
+                    formated_responders = default_responders
+            responders = formated_responders
+        return responders
+
+    def _fill_responders(self, responders, type_):
+        return [{'id': r, 'type': type_} for r in responders]
 
     def alert(self, matches):
         body = ''
         for match in matches:
-            body += unicode(BasicMatchString(self.rule, match))
+            body += str(BasicMatchString(self.rule, match))
             # Separate text of aggregated alerts with dashes
             if len(matches) > 1:
                 body += '\n----------------------------------------\n'
@@ -40,26 +71,38 @@ class OpsGenieAlerter(Alerter):
             self.message = self.create_title(matches)
         else:
             self.message = self.custom_message.format(**matches[0])
-
+        self.recipients = self._parse_responders(self.recipients, self.recipients_args, matches, self.default_reciepients)
+        self.teams = self._parse_responders(self.teams, self.teams_args, matches, self.default_teams)
         post = {}
-        post['apiKey'] = self.api_key
         post['message'] = self.message
         if self.account:
             post['user'] = self.account
         if self.recipients:
-            post['recipients'] = self.recipients
+            post['responders'] = [{'username': r, 'type': 'user'} for r in self.recipients]
         if self.teams:
-            post['teams'] = self.teams
+            post['teams'] = [{'name': r, 'type': 'team'} for r in self.teams]
         post['description'] = body
         post['source'] = 'ElastAlert'
+
+        for i, tag in enumerate(self.tags):
+            self.tags[i] = tag.format(**matches[0])
         post['tags'] = self.tags
 
+        if self.priority and self.priority not in ('P1', 'P2', 'P3', 'P4', 'P5'):
+            logging.warn("Priority level does not appear to be specified correctly. \
+                         Please make sure to set it to a value between P1 and P5")
+        else:
+            post['priority'] = self.priority
+
         if self.alias is not None:
             post['alias'] = self.alias.format(**matches[0])
 
         logging.debug(json.dumps(post))
 
-        headers = {'content-type': 'application/json'}
+        headers = {
+            'Content-Type': 'application/json',
+            'Authorization': 'GenieKey {}'.format(self.api_key),
+        }
         # set https proxy, if it was provided
         proxies = {'https': self.opsgenie_proxy} if self.opsgenie_proxy else None
 
@@ -67,7 +110,7 @@ class OpsGenieAlerter(Alerter):
             r = requests.post(self.to_addr, json=post, headers=headers, proxies=proxies)
 
             logging.debug('request response: {0}'.format(r))
-            if r.status_code != 200:
+            if r.status_code != 202:
                 elastalert_logger.info("Error response from {0} \n "
                                        "API Response: {1}".format(self.to_addr, r))
                 r.raise_for_status()
@@ -94,13 +137,13 @@ class OpsGenieAlerter(Alerter):
         return self.create_default_title(matches)
 
     def create_custom_title(self, matches):
-        opsgenie_subject = unicode(self.rule['opsgenie_subject'])
+        opsgenie_subject = str(self.rule['opsgenie_subject'])
 
         if self.opsgenie_subject_args:
             opsgenie_subject_values = [lookup_es_key(matches[0], arg) for arg in self.opsgenie_subject_args]
 
-            for i in xrange(len(opsgenie_subject_values)):
-                if opsgenie_subject_values[i] is None:
+            for i, subject_value in enumerate(opsgenie_subject_values):
+                if subject_value is None:
                     alert_value = self.rule.get(self.opsgenie_subject_args[i])
                     if alert_value:
                         opsgenie_subject_values[i] = alert_value
@@ -118,5 +161,4 @@ class OpsGenieAlerter(Alerter):
             ret['account'] = self.account
         if self.teams:
             ret['teams'] = self.teams
-
         return ret
